@@ -2,8 +2,11 @@ package Safe;
 
 use 5.003_11;
 use strict;
+use Scalar::Util qw(reftype);
+use Config qw(%Config);
+use constant is_usethreads => $Config{usethreads};
 
-$Safe::VERSION = "2.15";
+$Safe::VERSION = "2.20";
 
 # *** Don't declare any lexicals above this point ***
 #
@@ -46,7 +49,6 @@ my $default_root  = 0;
 my $default_share = [qw[
     *_
     &PerlIO::get_layers
-    &Regexp::DESTROY
     &UNIVERSAL::isa
     &UNIVERSAL::can
     &UNIVERSAL::VERSION
@@ -61,7 +63,9 @@ my $default_share = [qw[
     $version::VERSION
     $version::CLASS
     @version::ISA
-], ($] >= 5.010 && qw[
+], ($] >= 5.008001 && qw[
+    &Regexp::DESTROY
+]), ($] >= 5.010 && qw[
     &re::is_regexp
     &re::regname
     &re::regnames
@@ -242,13 +246,15 @@ sub share_from {
 	my ($var, $type);
 	$type = $1 if ($var = $arg) =~ s/^(\W)//;
 	# warn "share_from $pkg $type $var";
-	*{$root."::$var"} = (!$type)       ? \&{$pkg."::$var"}
+	for (1..2) { # assign twice to avoid any 'used once' warnings
+	    *{$root."::$var"} = (!$type)       ? \&{$pkg."::$var"}
 			  : ($type eq '&') ? \&{$pkg."::$var"}
 			  : ($type eq '$') ? \${$pkg."::$var"}
 			  : ($type eq '@') ? \@{$pkg."::$var"}
 			  : ($type eq '%') ? \%{$pkg."::$var"}
 			  : ($type eq '*') ?  *{$pkg."::$var"}
 			  : croak(qq(Can't share "$type$var" of unknown type));
+	}
     }
     $obj->share_record($pkg, $vars) unless $no_record or !$vars;
 }
@@ -285,8 +291,26 @@ sub reval {
     my ($obj, $expr, $strict) = @_;
     my $root = $obj->{Root};
 
-    my $evalsub = lexless_anon_sub($root,$strict, $expr);
-    return Opcode::_safe_call_sv($root, $obj->{Mask}, $evalsub);
+    my $evalsub = lexless_anon_sub($root, $strict, $expr);
+    my @ret = (wantarray)
+        ?        Opcode::_safe_call_sv($root, $obj->{Mask}, $evalsub)
+        : scalar Opcode::_safe_call_sv($root, $obj->{Mask}, $evalsub);
+
+    # RT#60374: Safe.pm sort {} bug with -Dusethreads
+    # If the Safe eval returns a code ref in a perl compiled with usethreads
+    # then wrap code ref with _safe_call_sv so that, when called, the
+    # execution will happen with the compartment fully 'in effect'.
+    # Needed to fix sort blocks that reference $a & $b and
+    # possibly other subtle issues.
+    if (is_usethreads()) {
+        for my $ret (@ret) { # edit (via alias) any CODE refs
+            next unless (reftype($ret)||'') eq 'CODE';
+            my $sub = $ret; # avoid closure problems
+            $ret = sub { Opcode::_safe_call_sv($root, $obj->{Mask}, $sub) };
+        }
+    }
+
+    return (wantarray) ? @ret : $ret[0];
 }
 
 sub rdo {
@@ -485,7 +509,7 @@ variable without any leading type marker. For example,
     ${$cpt->varglob('foo')} = "Hello world";
 
 
-=item reval (STRING)
+=item reval (STRING, STRICT)
 
 This evaluates STRING as perl code inside the compartment.
 
@@ -512,6 +536,10 @@ This behaviour differs from the beta distribution of the Safe extension
 where earlier versions of perl made it hard to mimic the return
 behaviour of the eval() command and the context was always scalar.
 
+The formerly undocumented STRICT argument sets strictness: if true
+'use strict;' is used, otherwise it uses 'no strict;'. B<Note>: if
+STRICT is omitted 'no strict;' is the default.
+
 Some points to note:
 
 If the entereval op is permitted then the code can use eval "..." to
